
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.text.DecimalFormat;
import java.util.Arrays;

// these imports are for hrtlib timing library available from
// http://www.javaworld.com/javaworld/javaqa/2003-01/01-qa-0110-timing.html
import com.vladium.utils.timing.ITimer;
import com.vladium.utils.timing.TimerFactory;

// ----------------------------------------------------------------------------
/**
 * A simple performance test driver for Java Object Serialization. See the
 * article for details. 
 * 
 * @author (C) <a href="http://www.javaworld.com/columns/jw-qna-index.shtml">Vlad Roubtsov</a>, 2003
 */
public class SerialVersionUIDTest
{
    // public: ................................................................
    
    public static void main (final String [] args)
        throws Exception
    {
        final DecimalFormat format = new DecimalFormat ("#.000");
        
        // create a couple of timers to keep track of object
        // serialization/deserialization:
        s_wtimer = TimerFactory.newTimer ();
        s_rtimer = TimerFactory.newTimer ();
          
        // this is my test object:      
        final TestClass testobj = new TestClass ();
        
        // ITimer/perftest() warmup:
        for (int i = 0; i < 3000; ++ i)
        {
            perftest (testobj);
            
            s_wtimer.getDuration ();            
            s_wtimer.reset ();
            s_rtimer.reset ();
        }
        
        final int repeats = 500;
        final double [] durations = new double [repeats];
        final Object [] holder = new Object [repeats];
        
        runGC (); // request GC to minimize possible timing disruptions
        
        for (int r = 0; r < repeats; ++ r)
        {
            Object clone = perftest (testobj);
                        
            holder [r] = clone; // retain this reference to minimize GC activity
            durations [r] = s_wtimer.getDuration () + s_rtimer.getDuration ();
            
            s_wtimer.reset ();
            s_rtimer.reset ();
        }
        
        // compute the average:
        double avg = 0.0;
        for (int i = 0; i < repeats; ++ i) avg += durations [i];
        avg /= repeats;
        
        // sort to get percentile metrics:
        Arrays.sort (durations);
        
        System.out.println ("[min/avg/95th percentile, ms] min: "
            + format.format (durations [0])
            + ", avg: " + format.format (avg)
            + ", 95%: " + format.format (durations [(95 * repeats)/100]));
        //System.out.println ("[percentiles, ms] 50%: " + format.format (durations [repeats/2]) + ", 75%: " + format.format (durations [(3 * repeats)/4]) + ", 95%: " + format.format (durations [(95 * repeats)/100]));        
    }
    
    // protected: .............................................................

    // package: ...............................................................
    
    // private: ...............................................................

    /*
     * This is the actual test method. It clones 'obj' by in-memory serialization.
     * Only the costs of writeObject()/readObject() are tallied.
     */
    private static Object perftest (final Object obj)
        throws IOException, ClassNotFoundException
    {
        s_out.reset ();
        ObjectOutputStream oout = new ObjectOutputStream (s_out);
        
        s_wtimer.start ();
        oout.writeObject (obj);
        s_wtimer.stop ();
        
        ObjectInputStream in = new ObjectInputStream (new ByteArrayInputStream (s_out.toByteArray ()));
        
        s_rtimer.start ();
        final Object result = in.readObject ();
        s_rtimer.stop ();
        
        return result;
    }
    
    private static ITimer s_wtimer, s_rtimer;
    private static final ByteArrayOutputStream s_out = new ByteArrayOutputStream (32 * 1024);
    

    // the code below was borrowed from Sizeof class
    // available from http://www.javaworld.com/javaworld/javatips/jw-javatip130.html
    private static void runGC () throws Exception
    {
        // for whatever reason it helps to call Runtime.gc()
        // using several method calls:
        for (int r = 0; r < 4; ++ r) _runGC ();
    }

    private static void _runGC () throws Exception
    {
        long usedMem1 = usedMemory (), usedMem2 = Long.MAX_VALUE;

        for (int i = 0; (usedMem1 < usedMem2) && (i < 1000); ++ i)
        {
            s_runtime.runFinalization ();
            s_runtime.gc ();
            Thread.currentThread ().yield ();
            
            usedMem2 = usedMem1;
            usedMem1 = usedMemory ();
        }
    }

    private static long usedMemory ()
    {
        return s_runtime.totalMemory () - s_runtime.freeMemory ();
    }
    
    private static final Runtime s_runtime = Runtime.getRuntime ();
    
} // end of class
// ----------------------------------------------------------------------------